Package: MBASED
Type: Package
Title: Package containing functions for ASE analysis using
        Meta-analysis Based Allele-Specific Expression Detection
Version: 1.46.0
Date: 2014-08-27
Author: Oleg Mayba, Houston Gilbert
Maintainer: Oleg Mayba <mayba.oleg@gene.com>
Description: The package implements MBASED algorithm for detecting
        allele-specific gene expression from RNA count data, where
        allele counts at individual loci (SNVs) are integrated into a
        gene-specific measure of ASE, and utilizes simulations to
        appropriately assess the statistical significance of observed
        ASE.
biocViews: Sequencing, GeneExpression, Transcription
Depends: RUnit, BiocGenerics, BiocParallel, GenomicRanges,
        SummarizedExperiment
Suggests: BiocStyle
License: Artistic-2.0
git_url: https://git.bioconductor.org/packages/MBASED
git_branch: RELEASE_3_23
git_last_commit: ca73ae8
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:42:18 UTC; biocbuild
Built: R 4.6.0; ; 2026-04-29 04:49:01 UTC; unix
